unit ConvertWide;

{*
Sjisutf16
Converting the text from Shift_JIS to UTF-16
Copyright 2020 Sogaya

Free Pascal Compiler 3.0.4

*}

{$I+}

interface

uses
  Windows, SysUtils, ErrorText, SparseUTF16, FinalEffect;

type ByteArray = array[0..223] of Byte;
type AnsiCharArray = array[0..4095] of AnsiChar;
type WideCharArray = array[0..4095] of WideChar;

type
  ConvertWiden = class
  private
    a: AnsiCharArray;
    A_SIZE: Integer;
    A_SIZE_1: Integer;
    beginning: Integer;
    BELL_ASCII_KATAKANA: ByteArray;
    buffer: array of AnsiChar;
    buffer_size: Integer;
    carriage_return_appears: Boolean;
    CLEAR_ARRAY: AnsiCharArray;
    CLEAR_V_ARRAY: WideCharArray;
    combo: array of AnsiChar;
    combo_size: Integer;
    count: Integer;
    EMPTY_ANSI: AnsiString;
    file_name: String;
    H_0A_C: AnsiChar;
    H_0A_W: WideChar;
    H_0D_C: AnsiChar;
    H_0D_W: WideChar;
    H_FE_C: AnsiChar;
    H_FF_C: AnsiChar;
    KANJI_ANSI: AnsiChar;
    NEW_LINE: AnsiString;
    NEW_LINE_SIZE: Integer;
    new_name: String;
    NON_KANJI_ANSI: AnsiChar;
    table: SparsePopulationUTF16;
    v: WideCharArray;
    V_SIZE: Integer;
    V_SIZE_1: Integer;
    x: ErrorTexts;
    function file_size(name: String): Integer;
    function setBufferSize: Integer;
  public
    destructor Destroy; override;
    function bufferingWriting(comboLength: Integer): Integer;
    function copyingArrayItem(index: Integer): WideChar;
    function displayDataText: Boolean;
    function lineCount: Integer;
    function newLine: AnsiString;
    function produce_UTF16_phrase(s: PAnsiChar; s_length: Integer): WideCharArray;
    function produceBinaryFile(name: String; no_removing: Boolean): Integer;
    function reading_to_produce_UTF16: WideCharArray;
    function reading_to_produce_UTF16_CRLF(nn: String): Integer;
    function reading_to_produce_UTF16_LF(nn: String): Integer;
    function reading_to_produce_UTF16_text(nn: String): Integer;
    function renameBinaryFile(name: String; no_removing: Boolean): Integer;
    function skipping_control_to_produce_UTF16_CRLF(nn: String): Integer;
    function skipping_control_to_produce_UTF16_LF(nn: String): Integer;
    function skipping_control_to_produce_UTF16_text(nn: String): Integer;
    function UTF16_phrase_size: Integer;
    procedure ifErrorOccurs(exit_code: Integer);
    procedure presetting;
    procedure setting(final_effects: FinalEffects); overload;
    procedure setting(final_effects: FinalEffects; b: Integer); overload;
    procedure setting(final_effects: FinalEffects; fn: String); overload;
  end;

const
  BELL = 7;
  BUFFER_ERROR = -1;
  C_ERR = 'err';
  COMBO_ERROR = -3;
  COMMA_BYTE = 44; { 2C, ',' }
  EXPANSION_FACTOR = 2;
  FILE_ERROR = -2;
  H_00FF = 255;
  H_0A = 10; { LF }
  H_0D = 13; { CR }
  H_5C = 92; { '\' }
  H_80 = 128;
  H_81 = 129;
  H_FC = 252;
  H_FE = 254;
  H_FF = 255;
  KANA_HANDAKUTEN = 223; { DFh }
  KANA_KUTEN = 161; { A1h }
  KANJI_SIZE = 2;
  SIZE_OF_ANSICHAR = 1;
  SIZE_OF_CHAR = 1;
  TEMPORARY_BINARY = 'binary.tmp';
  TEXT_FILE = '.txt';

implementation

destructor ConvertWiden.Destroy;
begin
  buffer := nil;
  combo := nil;
  table.Free;

  inherited Destroy;
end;

function ConvertWiden.bufferingWriting(comboLength: Integer): Integer;
var
  block_has_been_written: Integer;
  exit_code: Integer;
  k: Integer;
  n: Integer;
  newFile: File;
  s: String;
begin
  exit_code := 0;
  n := 0;

{ Writing }
  try
    AssignFile(newFile, new_name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;

      for k := 0 to A_SIZE_1 do
      begin
        a[k] := combo[n];
        Inc(n);
        if n >= comboLength then
          begin
            A_SIZE := k + 1;
            break;
          end;

      end;

      BlockWrite(newFile, a, A_SIZE, block_has_been_written);
    until n >= comboLength;

    CloseFile(newFile);
    A_SIZE := SizeOf(a);
  except
    s := 'Rewrite() error-70: ' + new_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    exit_code := FILE_ERROR;
  end;

  Result := exit_code;
end;

function ConvertWiden.copyingArrayItem(index: Integer): WideChar;
var
  r: WideChar;
begin
  if index < 0 then
    begin
      r := #0;
    end
  else
    begin
      if index > A_SIZE_1 then
        begin
          r := #0;
        end
      else
        begin
          r := v[index];
        end;

    end;

  Result := r;
end;

function ConvertWiden.displayDataText: Boolean;
var
  b: Boolean;
  s: String;
  testFile: TextFile;
begin
  b := True;

  try
    AssignFile(testFile, DATA_TEXT);
    Reset(testFile);

    while not Eof(testFile) do
    begin
      ReadLn(testFile, s);
      WriteLn(s);
    end;

    CloseFile(testFile);
  except
    s := 'Reset() error-71: ' + DATA_TEXT;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    b := False;
  end;

  Result := b;
end;

function ConvertWiden.file_size(name: String): Integer;
var
  f: Integer;
  size: Integer;
begin
  f := FileOpen(name, fmOpenRead or fmShareDenyNone);
  size := FileSeek(f, 0, 2); { EOF }
  FileClose(f);
  Result := size;
end;

function ConvertWiden.lineCount: Integer;
begin
  Result := count;
end;

function ConvertWiden.newLine: AnsiString;
begin
  Result := NEW_LINE;
end;

function ConvertWiden.produceBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, TEMPORARY_BINARY);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);
      BlockWrite(newFile, a, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, name);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-72: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function ConvertWiden.produce_UTF16_phrase(s: PAnsiChar; s_length: Integer): WideCharArray;
var
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  first_SJIS_byte: Integer;
  h: Integer;
  higher_byte_part_of_UTF16: AnsiChar;
  k: Longint;
  kanji: Boolean;
  lower_byte_part_of_UTF16: AnsiChar;
  m: Integer;
  n: Integer;
  s_length_1: Integer;
  SJIS_code: Integer;
  UTF16_code: Integer;
begin
  beginning := 0;
  carriage_return_appears := False;
  m := 0;
  s_length_1 := s_length - 1;
  v := CLEAR_V_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  first_SJIS_byte := Ord(firstAnsiChar);
  kanji := False;
  for n := 0 to s_length_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    c := s[n];
    k := Ord(c);

    if kanji then
      begin
        SJIS_code := first_SJIS_byte + k;

        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

        h := Ord(higher_byte_part_of_UTF16);
        h := h shl 8;
        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

        v[m] := table.UTF16Char(UTF16_code);
        Inc(m);

        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            v[m] := H_0A_W;
            Inc(m);

            beginning := n + 1;
            break;
          end
        else
          begin
            if k = H_0D then
              begin
                v[m] := H_0D_W;
                Inc(m);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        SJIS_code := k;

                        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
                        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

                        h := Ord(higher_byte_part_of_UTF16);
                        h := h shl 8;
                        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

                        v[m] := table.UTF16Char(UTF16_code);
                        Inc(m);

                        kanji := False;
                      end
                    else
                      begin
                        first_SJIS_byte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    v[m] := table.UTF16Char(k);
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := 0 to s_length_1 do }

  if beginning > 0 then
    begin
{ Counting lines }
      firstAnsiChar := Chr(COMMA_BYTE);
      first_SJIS_byte := Ord(firstAnsiChar);
      kanji := False;
      for n := beginning to s_length_1 do
      begin
        if m >= V_SIZE_1 then
          begin
            break;
          end;

        c := s[n];
        k := Ord(c);

        if kanji then
          begin
            SJIS_code := first_SJIS_byte + k;

            higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
            lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

            h := Ord(higher_byte_part_of_UTF16);
            h := h shl 8;
            UTF16_code := h + Ord(lower_byte_part_of_UTF16);

            v[m] := table.UTF16Char(UTF16_code);
            Inc(m);

            kanji := False;
          end
        else
          begin
            if k = H_0A then
              begin
                Inc(count);

                v[m] := H_0A_W;
                Inc(m);
              end
            else
              begin
                if k = H_0D then
                  begin
                    v[m] := H_0D_W;
                    Inc(m);
                  end
                else
                  begin
                    if (H_81 <= k) and (k <= H_FC) then
                      begin
                        if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                          begin
                            SJIS_code := k;

                            higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
                            lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

                            h := Ord(higher_byte_part_of_UTF16);
                            h := h shl 8;
                            UTF16_code := h + Ord(lower_byte_part_of_UTF16);

                            v[m] := table.UTF16Char(UTF16_code);
                            Inc(m);

                            kanji := False;
                          end
                        else
                          begin
                            first_SJIS_byte := k shl 8;
                            kanji := True;
                          end;

                      end
                    else
                      begin
{ ASCII }
                        v[m] := table.UTF16Char(k);
                        Inc(m);
                      end;

                  end;

              end; { if k = H_0A then, else }

          end; { if kanji then, else }

      end; { for n := beginning to s_length_1 do }

    end; { if beginning > 0 then }

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function ConvertWiden.reading_to_produce_UTF16: WideCharArray;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  first_SJIS_byte: Integer;
  h: Integer;
  higher_byte_part_of_UTF16: AnsiChar;
  k: Integer;
  kanji: Boolean;
  lower_byte_part_of_UTF16: AnsiChar;
  m: Integer;
  n: Integer;
  SJIS_code: Integer;
  s: String;
  testFile: File;
  UTF16_code: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  carriage_return_appears := False;
  k := 0;
  m := 0;
  v := CLEAR_V_ARRAY;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-73: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  first_SJIS_byte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        SJIS_code := first_SJIS_byte + k;

        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

        h := Ord(higher_byte_part_of_UTF16);
        h := h shl 8;
        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

        v[m] := table.UTF16Char(UTF16_code);
        Inc(m);

        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            v[m] := H_0A_W;
            Inc(m);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                v[m] := H_0D_W;
                Inc(m);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        SJIS_code := k;

                        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
                        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

                        h := Ord(higher_byte_part_of_UTF16);
                        h := h shl 8;
                        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

                        v[m] := table.UTF16Char(UTF16_code);
                        Inc(m);

                        kanji := False;
                      end
                    else
                      begin
                        first_SJIS_byte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    v[m] := table.UTF16Char(k);
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  first_SJIS_byte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    if m >= V_SIZE_1 then
      begin
        break;
      end;

    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        SJIS_code := first_SJIS_byte + k;

        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

        h := Ord(higher_byte_part_of_UTF16);
        h := h shl 8;
        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

        v[m] := table.UTF16Char(UTF16_code);
        Inc(m);

        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            v[m] := H_0A_W;
            Inc(m);
          end
        else
          begin
            if k = H_0D then
              begin
                v[m] := H_0D_W;
                Inc(m);
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        SJIS_code := k;

                        higher_byte_part_of_UTF16 := table.working(SJIS_code, 1);
                        lower_byte_part_of_UTF16 := table.working(SJIS_code, 0); { little-endian architecture }

                        h := Ord(higher_byte_part_of_UTF16);
                        h := h shl 8;
                        UTF16_code := h + Ord(lower_byte_part_of_UTF16);

                        v[m] := table.UTF16Char(UTF16_code);
                        Inc(m);

                        kanji := False;
                      end
                    else
                      begin
                        first_SJIS_byte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    v[m] := table.UTF16Char(k);
                    Inc(m);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  V_SIZE := m;
  V_SIZE_1 := V_SIZE - 1;
  Result := v;
end;

function ConvertWiden.reading_to_produce_UTF16_CRLF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  m: Integer;
  kanji: Boolean;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-74: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if k = H_0D then
              begin
                continue;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.reading_to_produce_UTF16_LF(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  m: Integer;
  kanji: Boolean;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-75: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if k = H_0D then
              begin
                continue; { Removing 0Dh }
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.reading_to_produce_UTF16_text(nn: String): Integer;
var
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  m: Integer;
  kanji: Boolean;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n := 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-76: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                combo[ m + 1 ] := #0;
                Inc(m, KANJI_SIZE);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                combo[ m + 1 ] := #0;
                Inc(m, KANJI_SIZE);
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.renameBinaryFile(name: String; no_removing: Boolean): Integer;
var
  block_has_been_read: Integer;
  block_has_been_written: Integer;
  newFile: File;
  s: String;
  sum: Integer;
  testFile: File;
begin
  sum := 0;

  try
    AssignFile(testFile, TEMPORARY_BINARY);
    Reset(testFile, SIZE_OF_ANSICHAR);

    AssignFile(newFile, name);
    Rewrite(newFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);
      BlockWrite(newFile, a, block_has_been_read, block_has_been_written);
      sum := sum + block_has_been_written;
    until (block_has_been_read = 0) or (block_has_been_read <> block_has_been_written);

    CloseFile(newFile);
    CloseFile(testFile);

    if not no_removing then
      begin
        AssignFile(testFile, TEMPORARY_BINARY);
        Erase(testFile);
      end;

  except
    s := 'Reset() error-77: ' + name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    Halt(1);
  end;

  Result := sum;
end;

function ConvertWiden.setBufferSize: Integer;
var
  r: Integer;
  s: String;
begin
  r := buffer_size;

  try
    SetLength(buffer, buffer_size);
  except
    s := 'SetLength() error-78: ' + IntToStr(buffer_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := BUFFER_ERROR;
  end;

  try
    SetLength(combo, combo_size);
  except
    s := 'SetLength() error-79: ' + IntToStr(combo_size);
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
    r := COMBO_ERROR;
  end;

  Result := r;
end;

function ConvertWiden.skipping_control_to_produce_UTF16_CRLF(nn: String): Integer;
var
  b: Integer;
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-80: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        combo[ m + 1 ] := #0;
                        Inc(m, KANJI_SIZE);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0D_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if k = H_0D then
              begin
                continue;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        combo[ m + 1 ] := #0;
                        Inc(m, KANJI_SIZE);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.skipping_control_to_produce_UTF16_LF(nn: String): Integer;
var
  b: Byte;
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-81: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if (H_81 <= k) and (k <= H_FC) then
              begin
                if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                  begin
                    combo[m] := table.working(k, 0);
                    combo[ m + 1 ] := table.working(k, 1);
                    Inc(m, KANJI_SIZE);
                    kanji := False;
                  end
                else
                  begin
                    firstByte := k shl 8;
                    kanji := True;
                  end;

              end
            else
              begin
{ ASCII }
                b := BELL_ASCII_KATAKANA[k];
                if b = BELL then
                  begin
                    continue; { Removing control characters }
                  end
                else
                  begin
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if (H_81 <= k) and (k <= H_FC) then
              begin
                if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                  begin
                    combo[m] := table.working(k, 0);
                    combo[ m + 1 ] := table.working(k, 1);
                    Inc(m, KANJI_SIZE);
                    kanji := False;
                  end
                else
                  begin
                    firstByte := k shl 8;
                    kanji := True;
                  end;

              end
            else
              begin
{ ASCII }
                b := BELL_ASCII_KATAKANA[k];
                if b = BELL then
                  begin
                    continue; { Removing control characters }
                  end
                else
                  begin
                    combo[m] := c;
                    combo[ m + 1 ] := #0;
                    Inc(m, KANJI_SIZE);
                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.skipping_control_to_produce_UTF16_text(nn: String): Integer;
var
  b: Integer;
  block_has_been_read: Integer;
  buffer_size_1: Integer;
  c: AnsiChar;
  firstAnsiChar: AnsiChar;
  firstByte: Integer;
  k: Integer;
  kanji: Boolean;
  m: Integer;
  n: Integer;
  s: String;
  testFile: File;
  w: Integer;
begin
  buffer_size_1 := buffer_size - 1;
  k := 0;
  m := 0;
  new_name := nn;
  combo[m] := H_FF_C;
  combo[ m + 1 ] := H_FE_C;
  Inc(m, KANJI_SIZE);

  for n := 0 to buffer_size_1 do
  begin
    buffer[n] := #0;
  end;

  try
{ Reading a file }
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);

    repeat
      a := CLEAR_ARRAY;
      BlockRead(testFile, a, A_SIZE, block_has_been_read);

      for n:= 0 to A_SIZE_1 do
      begin
        buffer[k] := a[n];
        Inc(k);
        if k > buffer_size_1 then
          break;

      end;
    until k > buffer_size_1;

    CloseFile(testFile);
  except
    s := 'Reset() error-82: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

{ Finding the first NEW_LINE }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    beginning := n + 1;
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            count := 1;

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);

            break;
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                combo[ m + 1 ] := #0;
                Inc(m, KANJI_SIZE);

                carriage_return_appears := True;
                NEW_LINE_SIZE := 2; { 0Dh and 0Ah }
                NEW_LINE := EMPTY_ANSI + H_0D_C;
                NEW_LINE := NEW_LINE + H_0A_C;
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        combo[ m + 1 ] := #0;
                        Inc(m, KANJI_SIZE);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

{ Counting lines }
  firstAnsiChar := Chr(COMMA_BYTE);
  firstByte := Ord(firstAnsiChar);
  kanji := False;
  for n := beginning to buffer_size_1 do
  begin
    c := buffer[n];
    k := Ord(c);

    if kanji then
      begin
        w := firstByte + k;

        combo[m] := table.working(w, 0);
        combo[ m + 1 ] := table.working(w, 1);
        Inc(m, KANJI_SIZE);
        kanji := False;
      end
    else
      begin
        if k = H_0A then
          begin
            Inc(count);

            combo[m] := H_0A_C;
            combo[ m + 1 ] := #0;
            Inc(m, KANJI_SIZE);
          end
        else
          begin
            if k = H_0D then
              begin
                combo[m] := H_0D_C;
                combo[ m + 1 ] := #0;
                Inc(m, KANJI_SIZE);
              end
            else
              begin
                if (H_81 <= k) and (k <= H_FC) then
                  begin
                    if (KANA_KUTEN <= k) and (k <= KANA_HANDAKUTEN) then
                      begin
                        combo[m] := table.working(k, 0);
                        combo[ m + 1 ] := table.working(k, 1);
                        Inc(m, KANJI_SIZE);
                        kanji := False;
                      end
                    else
                      begin
                        firstByte := k shl 8;
                        kanji := True;
                      end;

                  end
                else
                  begin
{ ASCII }
                    b := BELL_ASCII_KATAKANA[k];
                    if b = BELL then
                      begin
                        continue; { Removing control characters }
                      end
                    else
                      begin
                        combo[m] := c;
                        combo[ m + 1 ] := #0;
                        Inc(m, KANJI_SIZE);
                      end;

                  end;

              end;

          end; { if k = H_0A then, else }

      end; { if kanji then, else }

  end; { for n := beginning to buffer_size_1 do }

  Result := bufferingWriting(m);
end;

function ConvertWiden.UTF16_phrase_size: Integer;
begin
  Result := V_SIZE;
end;

procedure ConvertWiden.ifErrorOccurs(exit_code: Integer);
begin
  if buffer_size < 0 then
    Halt(exit_code);

end;

procedure ConvertWiden.presetting;
begin
  BELL_ASCII_KATAKANA[0] := 7; BELL_ASCII_KATAKANA[1] := 7; BELL_ASCII_KATAKANA[2] := 7; BELL_ASCII_KATAKANA[3] := 7; BELL_ASCII_KATAKANA[4] := 7; BELL_ASCII_KATAKANA[5] := 7;
  BELL_ASCII_KATAKANA[6] := 7; BELL_ASCII_KATAKANA[7] := 7; BELL_ASCII_KATAKANA[8] := 7; BELL_ASCII_KATAKANA[9] := 7; BELL_ASCII_KATAKANA[10] := 7; BELL_ASCII_KATAKANA[11] := 7;
  BELL_ASCII_KATAKANA[12] := 7; BELL_ASCII_KATAKANA[13] := 7; BELL_ASCII_KATAKANA[14] := 7; BELL_ASCII_KATAKANA[15] := 7; BELL_ASCII_KATAKANA[16] := 7; BELL_ASCII_KATAKANA[17] := 7;
  BELL_ASCII_KATAKANA[18] := 7; BELL_ASCII_KATAKANA[19] := 7; BELL_ASCII_KATAKANA[20] := 7; BELL_ASCII_KATAKANA[21] := 7; BELL_ASCII_KATAKANA[22] := 7; BELL_ASCII_KATAKANA[23] := 7;
  BELL_ASCII_KATAKANA[24] := 7; BELL_ASCII_KATAKANA[25] := 7; BELL_ASCII_KATAKANA[26] := 7; BELL_ASCII_KATAKANA[27] := 7; BELL_ASCII_KATAKANA[28] := 7; BELL_ASCII_KATAKANA[29] := 7;
  BELL_ASCII_KATAKANA[30] := 7; BELL_ASCII_KATAKANA[31] := 7; BELL_ASCII_KATAKANA[32] := 32; BELL_ASCII_KATAKANA[33] := 33; BELL_ASCII_KATAKANA[34] := 34; BELL_ASCII_KATAKANA[35] := 35;
  BELL_ASCII_KATAKANA[36] := 36; BELL_ASCII_KATAKANA[37] := 37; BELL_ASCII_KATAKANA[38] := 38; BELL_ASCII_KATAKANA[39] := 39; BELL_ASCII_KATAKANA[40] := 40; BELL_ASCII_KATAKANA[41] := 41;
  BELL_ASCII_KATAKANA[42] := 42; BELL_ASCII_KATAKANA[43] := 43; BELL_ASCII_KATAKANA[44] := 44; BELL_ASCII_KATAKANA[45] := 45; BELL_ASCII_KATAKANA[46] := 46; BELL_ASCII_KATAKANA[47] := 47;
  BELL_ASCII_KATAKANA[48] := 48; BELL_ASCII_KATAKANA[49] := 49; BELL_ASCII_KATAKANA[50] := 50; BELL_ASCII_KATAKANA[51] := 51; BELL_ASCII_KATAKANA[52] := 52; BELL_ASCII_KATAKANA[53] := 53;
  BELL_ASCII_KATAKANA[54] := 54; BELL_ASCII_KATAKANA[55] := 55; BELL_ASCII_KATAKANA[56] := 56; BELL_ASCII_KATAKANA[57] := 57; BELL_ASCII_KATAKANA[58] := 58; BELL_ASCII_KATAKANA[59] := 59;
  BELL_ASCII_KATAKANA[60] := 60; BELL_ASCII_KATAKANA[61] := 61; BELL_ASCII_KATAKANA[62] := 62; BELL_ASCII_KATAKANA[63] := 63; BELL_ASCII_KATAKANA[64] := 64; BELL_ASCII_KATAKANA[65] := 65;
  BELL_ASCII_KATAKANA[66] := 66; BELL_ASCII_KATAKANA[67] := 67; BELL_ASCII_KATAKANA[68] := 68; BELL_ASCII_KATAKANA[69] := 69; BELL_ASCII_KATAKANA[70] := 70; BELL_ASCII_KATAKANA[71] := 71;
  BELL_ASCII_KATAKANA[72] := 72; BELL_ASCII_KATAKANA[73] := 73; BELL_ASCII_KATAKANA[74] := 74; BELL_ASCII_KATAKANA[75] := 75; BELL_ASCII_KATAKANA[76] := 76; BELL_ASCII_KATAKANA[77] := 77;
  BELL_ASCII_KATAKANA[78] := 78; BELL_ASCII_KATAKANA[79] := 79; BELL_ASCII_KATAKANA[80] := 80; BELL_ASCII_KATAKANA[81] := 81; BELL_ASCII_KATAKANA[82] := 82; BELL_ASCII_KATAKANA[83] := 83;
  BELL_ASCII_KATAKANA[84] := 84; BELL_ASCII_KATAKANA[85] := 85; BELL_ASCII_KATAKANA[86] := 86; BELL_ASCII_KATAKANA[87] := 87; BELL_ASCII_KATAKANA[88] := 88; BELL_ASCII_KATAKANA[89] := 89; 
  BELL_ASCII_KATAKANA[90] := 90; BELL_ASCII_KATAKANA[91] := 91; BELL_ASCII_KATAKANA[92] := 92; BELL_ASCII_KATAKANA[93] := 93; BELL_ASCII_KATAKANA[94] := 94; BELL_ASCII_KATAKANA[95] := 95; 
  BELL_ASCII_KATAKANA[96] := 96; BELL_ASCII_KATAKANA[97] := 97; BELL_ASCII_KATAKANA[98] := 98; BELL_ASCII_KATAKANA[99] := 99; BELL_ASCII_KATAKANA[100] := 100; BELL_ASCII_KATAKANA[101] := 101; 
  BELL_ASCII_KATAKANA[102] := 102; BELL_ASCII_KATAKANA[103] := 103; BELL_ASCII_KATAKANA[104] := 104; BELL_ASCII_KATAKANA[105] := 105; BELL_ASCII_KATAKANA[106] := 106; BELL_ASCII_KATAKANA[107] := 107; 
  BELL_ASCII_KATAKANA[108] := 108; BELL_ASCII_KATAKANA[109] := 109; BELL_ASCII_KATAKANA[110] := 110; BELL_ASCII_KATAKANA[111] := 111; BELL_ASCII_KATAKANA[112] := 112; BELL_ASCII_KATAKANA[113] := 113; 
  BELL_ASCII_KATAKANA[114] := 114; BELL_ASCII_KATAKANA[115] := 115; BELL_ASCII_KATAKANA[116] := 116; BELL_ASCII_KATAKANA[117] := 117; BELL_ASCII_KATAKANA[118] := 118; BELL_ASCII_KATAKANA[119] := 119; 
  BELL_ASCII_KATAKANA[120] := 120; BELL_ASCII_KATAKANA[121] := 121; BELL_ASCII_KATAKANA[122] := 122; BELL_ASCII_KATAKANA[123] := 123; BELL_ASCII_KATAKANA[124] := 124; BELL_ASCII_KATAKANA[125] := 125; 
  BELL_ASCII_KATAKANA[126] := 126; BELL_ASCII_KATAKANA[127] := 7; BELL_ASCII_KATAKANA[128] := 7; BELL_ASCII_KATAKANA[129] := 7; BELL_ASCII_KATAKANA[130] := 7; BELL_ASCII_KATAKANA[131] := 7; 
  BELL_ASCII_KATAKANA[132] := 7; BELL_ASCII_KATAKANA[133] := 7; BELL_ASCII_KATAKANA[134] := 7; BELL_ASCII_KATAKANA[135] := 7; BELL_ASCII_KATAKANA[136] := 7; BELL_ASCII_KATAKANA[137] := 7; 
  BELL_ASCII_KATAKANA[138] := 7; BELL_ASCII_KATAKANA[139] := 7; BELL_ASCII_KATAKANA[140] := 7; BELL_ASCII_KATAKANA[141] := 7; BELL_ASCII_KATAKANA[142] := 7; BELL_ASCII_KATAKANA[143] := 7; 
  BELL_ASCII_KATAKANA[144] := 7; BELL_ASCII_KATAKANA[145] := 7; BELL_ASCII_KATAKANA[146] := 7; BELL_ASCII_KATAKANA[147] := 7; BELL_ASCII_KATAKANA[148] := 7; BELL_ASCII_KATAKANA[149] := 7; 
  BELL_ASCII_KATAKANA[150] := 7; BELL_ASCII_KATAKANA[151] := 7; BELL_ASCII_KATAKANA[152] := 7; BELL_ASCII_KATAKANA[153] := 7; BELL_ASCII_KATAKANA[154] := 7; BELL_ASCII_KATAKANA[155] := 7; 
  BELL_ASCII_KATAKANA[156] := 7; BELL_ASCII_KATAKANA[157] := 7; BELL_ASCII_KATAKANA[158] := 7; BELL_ASCII_KATAKANA[159] := 7; BELL_ASCII_KATAKANA[160] := 7; BELL_ASCII_KATAKANA[161] := 161; 
  BELL_ASCII_KATAKANA[162] := 162; BELL_ASCII_KATAKANA[163] := 163; BELL_ASCII_KATAKANA[164] := 164; BELL_ASCII_KATAKANA[165] := 165; BELL_ASCII_KATAKANA[166] := 166; BELL_ASCII_KATAKANA[167] := 167; 
  BELL_ASCII_KATAKANA[168] := 168; BELL_ASCII_KATAKANA[169] := 169; BELL_ASCII_KATAKANA[170] := 170; BELL_ASCII_KATAKANA[171] := 171; BELL_ASCII_KATAKANA[172] := 172; BELL_ASCII_KATAKANA[173] := 173; 
  BELL_ASCII_KATAKANA[174] := 174; BELL_ASCII_KATAKANA[175] := 175; BELL_ASCII_KATAKANA[176] := 176; BELL_ASCII_KATAKANA[177] := 177; BELL_ASCII_KATAKANA[178] := 178; BELL_ASCII_KATAKANA[179] := 179; 
  BELL_ASCII_KATAKANA[180] := 180; BELL_ASCII_KATAKANA[181] := 181; BELL_ASCII_KATAKANA[182] := 182; BELL_ASCII_KATAKANA[183] := 183; BELL_ASCII_KATAKANA[184] := 184; BELL_ASCII_KATAKANA[185] := 185; 
  BELL_ASCII_KATAKANA[186] := 186; BELL_ASCII_KATAKANA[187] := 187; BELL_ASCII_KATAKANA[188] := 188; BELL_ASCII_KATAKANA[189] := 189; BELL_ASCII_KATAKANA[190] := 190; BELL_ASCII_KATAKANA[191] := 191; 
  BELL_ASCII_KATAKANA[192] := 192; BELL_ASCII_KATAKANA[193] := 193; BELL_ASCII_KATAKANA[194] := 194; BELL_ASCII_KATAKANA[195] := 195; BELL_ASCII_KATAKANA[196] := 196; BELL_ASCII_KATAKANA[197] := 197; 
  BELL_ASCII_KATAKANA[198] := 198; BELL_ASCII_KATAKANA[199] := 199; BELL_ASCII_KATAKANA[200] := 200; BELL_ASCII_KATAKANA[201] := 201; BELL_ASCII_KATAKANA[202] := 202; BELL_ASCII_KATAKANA[203] := 203; 
  BELL_ASCII_KATAKANA[204] := 204; BELL_ASCII_KATAKANA[205] := 205; BELL_ASCII_KATAKANA[206] := 206; BELL_ASCII_KATAKANA[207] := 207; BELL_ASCII_KATAKANA[208] := 208; BELL_ASCII_KATAKANA[209] := 209; 
  BELL_ASCII_KATAKANA[210] := 210; BELL_ASCII_KATAKANA[211] := 211; BELL_ASCII_KATAKANA[212] := 212; BELL_ASCII_KATAKANA[213] := 213; BELL_ASCII_KATAKANA[214] := 214; BELL_ASCII_KATAKANA[215] := 215; 
  BELL_ASCII_KATAKANA[216] := 216; BELL_ASCII_KATAKANA[217] := 217; BELL_ASCII_KATAKANA[218] := 218; BELL_ASCII_KATAKANA[219] := 219; BELL_ASCII_KATAKANA[220] := 220; BELL_ASCII_KATAKANA[221] := 221; 
  BELL_ASCII_KATAKANA[222] := 222; BELL_ASCII_KATAKANA[223] := 223;
end;

procedure ConvertWiden.setting(final_effects: FinalEffects);
var
  i: Integer;
begin
{ Setting for converting phrases with no extra memory space }
  buffer_size := 0;
  combo_size := 0;

  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  file_name := '';
  new_name := '';
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  count := -1;
  EMPTY_ANSI := '';
  KANJI_ANSI := #1;
  NON_KANJI_ANSI := #0;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
    CLEAR_V_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := Chr(H_FF);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_FE_C := Chr(H_FE);

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

{ Producing a conversion table in the main memory }
  table := SparsePopulationUTF16.Create;
  table.setting(final_effects);
  H_0A_W := table.UTF16Char(H_0A);
  H_0D_W := table.UTF16Char(H_0D);
end;

procedure ConvertWiden.setting(final_effects: FinalEffects; b: Integer);
var
  i: Integer;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  buffer_size := b;
  combo_size := EXPANSION_FACTOR * buffer_size;
  file_name := '';
  new_name := '';
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  count := -1;
  EMPTY_ANSI := '';
  KANJI_ANSI := #1;
  NON_KANJI_ANSI := #0;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
    CLEAR_V_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := Chr(H_FF);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_FE_C := Chr(H_FE);

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationUTF16.Create;
  table.setting(final_effects);
  H_0A_W := table.UTF16Char(H_0A);
  H_0D_W := table.UTF16Char(H_0D);
end;

procedure ConvertWiden.setting(final_effects: FinalEffects; fn: String);
var
  i: Integer;
  s: String;
  testFile: File;
begin
  A_SIZE := SizeOf(a);
  A_SIZE_1 := A_SIZE - 1;
  beginning := 0;
  buffer_size := -1;
  file_name := fn;
  V_SIZE := SizeOf(v);
  V_SIZE_1 := V_SIZE - 1;

  try
    AssignFile(testFile, file_name);
    Reset(testFile, SIZE_OF_ANSICHAR);
    CloseFile(testFile);

    buffer_size := file_size(file_name);
    combo_size := EXPANSION_FACTOR * buffer_size;
  except
    s := 'Reset() error-83: ' + file_name;
    x := ErrorTexts.Create;
    x.setting(C_ERR, TEXT_FILE, s);
    x.Free;
  end;

  count := -1;
  EMPTY_ANSI := '';
  KANJI_ANSI := #1;
  NON_KANJI_ANSI := #0;

  for i := 0 to A_SIZE_1 do
  begin
    CLEAR_ARRAY[i] := #0;
    CLEAR_V_ARRAY[i] := #0;
  end;

{ Characters of AnsiChar type }
  H_FF_C := Chr(H_FF);
  H_0A_C := Chr(H_0A);
  H_0D_C := Chr(H_0D);
  H_FE_C := Chr(H_FE);

  NEW_LINE := EMPTY_ANSI + H_0A_C;
  NEW_LINE_SIZE := 1;

  setBufferSize;

{ Producing a conversion table in the main memory }
  table := SparsePopulationUTF16.Create;
  table.setting(final_effects);
  H_0A_W := table.UTF16Char(H_0A);
  H_0D_W := table.UTF16Char(H_0D);
end;

end.
